module ClearDynamicAttributes
  # 插件信息
  PLUGIN_NAME = '清除动态属性'
  PLUGIN_VERSION = '1.0.0'
  PLUGIN_AUTHOR = '刘涛'
  PLUGIN_COPYRIGHT = '© 2025 liutao91.cn'
  PLUGIN_DESCRIPTION = '可清除选定对象（包括嵌套组或组件）的动态属性。'

  # 插件初始化
  def self.init
    # 注册插件
    extension = SketchupExtension.new(PLUGIN_NAME, 'T_Clear_Dynamic_Attributes.rb')
    extension.version = PLUGIN_VERSION
    extension.creator = PLUGIN_AUTHOR
    extension.description = PLUGIN_DESCRIPTION
    extension.copyright = PLUGIN_COPYRIGHT
    Sketchup.register_extension(extension, true)

    # 创建工具栏
    toolbar = UI::Toolbar.new(PLUGIN_NAME)

    # 创建工具栏按钮
    cmd = UI::Command.new(PLUGIN_NAME) {
      self.clear_dynamic_attributes_in_selection_with_check
    }
    cmd.tooltip = '清除动态属性'
    cmd.status_bar_text = "清除选中项的所有动态属性。"

    # 设置按钮的图标
    cmd.small_icon = "T_Clear_Dynamic_Attributes/i.png"
    cmd.large_icon = "T_Clear_Dynamic_Attributes/i.png"

    # 添加按钮到工具栏
    toolbar.add_item(cmd)
    toolbar.show
  end

  # 清除动态属性操作前的检查
  def self.clear_dynamic_attributes_in_selection_with_check
    model = Sketchup.active_model
    selection = model.selection

    if selection.empty?
      UI.messagebox('没有选中任何对象，请选择一个或多个对象。')
      return
    end

    # 检查选中的对象是否包含 dynamic_attributes
    has_dynamic_attributes = false
    selection.each do |entity|
      if self.entity_has_dynamic_attributes?(entity)
        has_dynamic_attributes = true
        break
      end
    end

    if has_dynamic_attributes
      # 执行唯一化和清除操作
      self.make_entities_unique_in_selection(selection)
      self.clear_dynamic_attributes_in_selection
    else
      UI.messagebox('选中的对象没有包含任何动态属性。')
    end
  end

  # 将选中的对象及其嵌套实体设为唯一
  def self.make_entities_unique_in_selection(entities)
    # 避免重复操作，记住已经处理过的实体
    processed_entities = Set.new

    entities.each do |entity|
      self.make_entity_unique(entity, processed_entities)
    end
  end

  # 将实体及其嵌套实体设为唯一
  def self.make_entity_unique(entity, processed_entities)
    return if processed_entities.include?(entity)

    begin
      # 对组件实例进行唯一化处理
      if entity.is_a?(Sketchup::ComponentInstance)
        unless entity.definition.is_a?(Sketchup::ComponentDefinition) && entity.definition.instances.count == 1
          # 如果组件实例未被唯一化，则进行唯一化
          entity.make_unique
        end
      end

      # 对群组进行唯一化处理
      if entity.is_a?(Sketchup::Group)
        # 对群组直接调用 make_unique 进行唯一化
        entity.make_unique
      end

      # 将当前实体标记为已经处理过
      processed_entities.add(entity)

      # 递归处理组件或群组的嵌套实体
      if entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
        entity.definition.entities.each do |sub_entity|
          self.make_entity_unique(sub_entity, processed_entities)
        end
      end
    rescue => e
      # 捕获任何错误并打印到控制台，避免插件崩溃
      puts "错误: #{e.message}"
    end
  end

  # 检查实体是否包含 dynamic_attributes
  def self.entity_has_dynamic_attributes?(entity)
    # 如果实体是组件实例或群组，检查其自身及嵌套实体
    if entity.is_a?(Sketchup::ComponentInstance) || entity.is_a?(Sketchup::Group)
      return entity_has_dynamic_attributes_in_definition(entity) || entity_has_dynamic_attributes_in_entities(entity)
    end
    false
  end

  # 检查组件定义或群组定义中的动态属性
  def self.entity_has_dynamic_attributes_in_definition(entity)
    if entity.is_a?(Sketchup::ComponentInstance)
      return self.entity_has_dynamic_attributes?(entity.definition)
    elsif entity.is_a?(Sketchup::Group)
      return self.entity_has_dynamic_attributes?(entity.definition)
    end
    false
  end

  # 检查实体（组件或群组）及其嵌套实体中的动态属性
  def self.entity_has_dynamic_attributes_in_entities(entity)
    # 确保 attribute_dictionaries 存在且包含 dynamic_attributes
    if entity.respond_to?(:attribute_dictionaries)
      dictionaries = entity.attribute_dictionaries
      if dictionaries && dictionaries['dynamic_attributes']
        return true
      end
    end

    # 遍历嵌套的实体
    if entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
      entity.definition.entities.each do |sub_entity|
        if self.entity_has_dynamic_attributes?(sub_entity)
          return true
        end
      end
    end
    false
  end

  # 清除选中组或组件以及嵌套组/组件的所有 dynamic_attributes
  def self.clear_dynamic_attributes(entity)
    if entity.respond_to?(:attribute_dictionaries)
      dict_name = 'dynamic_attributes'
      dictionaries = entity.attribute_dictionaries

      if dictionaries && dictionaries[dict_name]
        dictionaries[dict_name].each do |key, value|
          entity.delete_attribute(dict_name, key)
        end
      end
    end

    # 如果实体是组件实例，递归清除其定义中的 dynamic_attributes
    if entity.is_a?(Sketchup::ComponentInstance)
      component_definition = entity.definition
      self.clear_dynamic_attributes(component_definition)
    end

    # 如果实体是组或组件，递归清除其嵌套的组/组件的动态属性
    if entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
      entity.definition.entities.each do |sub_entity|
        self.clear_dynamic_attributes(sub_entity)
      end
    end
  end

  # 使用 start_operation 和 commit_operation 来提高效率
  def self.clear_dynamic_attributes_in_selection
    model = Sketchup.active_model
    selection = model.selection

    model.start_operation('清除动态属性', true)

    # 遍历选中的实体并清除其动态属性
    selection.each do |entity|
      self.clear_dynamic_attributes(entity)
    end

    model.commit_operation

    UI.messagebox('选中项的所有动态属性已被清除！')
  end
end

# 初始化插件
ClearDynamicAttributes.init
